/* 
	Context - software for storing, publishing and receiving context information
	on Nokia Series 60 version 1 and version 2 smartphones

    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#include "locationing.h"
#include <PLPVAR.H>
#include <PLPVARIANT.H>

#include "cl_settings.h"

#ifndef __WINS__
#define WAIT_TIME	90
#else
#define WAIT_TIME	1
#endif

#ifndef __WINS__
#define WAIT_TIME_DISCONNECTED	90
#else
#define WAIT_TIME_DISCONNECTED	1
#endif 

void Clocationing::ConstructL(i_status_notif* i_cb, MNaming* i_naming, sms* i_sms_handler, CPresencePublisher * aPublisher)
{
	CALLSTACKITEM(_L("Clocationing::ConstructL"));

	iName=(TText*)L"Clocationing";	
	iNaming=i_naming;
	cb=i_cb;
	
	smsh = i_sms_handler;

	iPublisher = aPublisher;

	iTimer=CTimeOut::NewL(*this);

	TInt enable;
	if (Settings().GetSettingL(SETTING_LOCATIONSERVICE_ENABLE, enable)) {
		iEnabled=enable;
	} else {
		iEnabled=true;
	}
#ifndef __WINS__
	TPlpVariantMachineId machineId;
	PlpVariant::GetMachineIdL(machineId);
	imsi=machineId;
#else
	// Return a fake IMEI when working on emulator
	_LIT(KEmulatorImsi, "244050000000000");
	imsi.Copy(KEmulatorImsi);
#endif
	smsh->AddHandler(this);

	Settings().NotifyOnChange(SETTING_LOCATIONSERVICE_ENABLE, this);

}

Clocationing::Clocationing(MApp_context& Context) : MContextBase(Context)
{
	CALLSTACKITEM(_L("Clocationing::Clocationing"));

}

Clocationing* Clocationing::NewL(MApp_context& Context, i_status_notif* i_cb, MNaming* i_naming, sms * i_sms_handler, CPresencePublisher * aPublisher)
{
	CALLSTACKITEMSTATIC(_L("Clocationing::NewL"));

	auto_ptr<Clocationing> ret(new (ELeave) Clocationing(Context));

	ret->ConstructL(i_cb, i_naming, i_sms_handler, aPublisher);
	return ret.release();
}

void Clocationing::handle_error(const TDesC& descr)
{
	CALLSTACKITEM(_L("Clocationing::handle_error"));
	
	// pending sms location reception
	iTimer->Reset();

	//resume presence
	iPublisher->ResumeConnection();

	cb->error(descr);
}

void Clocationing::handle_move(const TMsvId& /*msg_id*/, const TMsvId& /*from_folder*/, const TMsvId& /*to_folder*/, const TDesC& /*sender*/)
{
}

void Clocationing::handle_delete(const TMsvId& /*msg_id*/, const TMsvId& /*parent_folder*/, const TDesC& /*sender*/)
{
}

void Clocationing::handle_sending(const TMsvId& /*entry_id*/, const TDesC& /*sender*/, const TDesC & /*body*/)
{
}
void Clocationing::handle_change(const TMsvId& /*msg_id*/, const TDesC& /*sender*/)
{
}

void Clocationing::handle_read(const TMsvId& /*msg_id*/, const TDesC& /*sender*/)
{
}


bool Clocationing::handle_reception(const TMsvId& /*entry_id*/, const TMsvId& /*folder_id*/, const TDesC& sender, const TDesC& body)
{
	CALLSTACKITEM(_L("Clocationing::handle_reception"));

	if (sender.Compare(_L("16507")) ) {
		//cb->status_change(sender);
		return false;
	}
	
    iTimer->Reset();

	if (iToDiscard) {
		--iToDiscard;
		cb->status_change(_L("discarded unreliable result"));
		return true;
	}
	if (!iToName.SentRequest) return false;

#ifndef __WINS__
	/*
	 * FIXME: why was this done?
	 *
	TTime comp;
	comp.HomeTime();
	comp-=TTimeIntervalSeconds(WAIT_TIME);
	if (iToName.Time<comp) {
		iToName.Reset();
		cb->error(_L("Result too old"));
		return true;
	}
	*/
#endif
	
	if (body.Left(8).Compare(_L("itse  on"))) 
	{
		if (true || iRetryCount>5) 
		{
			/*
			 * It seems that if the locationing doesn't
			 * work, it won't work again before a cell change.
			 * The Radiolinja service probably sometimes doesn't
			 * get updated correctly, and doesn't retry.
			 * It's not worth trying again before we go
			 * to a new cell.
			 */
			iToName.Reset();
			cb->error(_L("Error result"));

			//resume presence
			iPublisher->ResumeConnection();
		} 
		else 
		{
			iToName.SentRequest=false;
			iRetryCount++;
			iTimer->Wait(WAIT_TIME*4);
		}
		
		return true;
	}

	HBufC* content=body.Mid(19, body.Length()-19-1).AllocLC();
	content->Des().Trim();

	cb->status_change(*content);

	iNaming->add_cellid_name(iToName.CellId, *content);
	CleanupStack::PopAndDestroy(); //content
	iToName.Reset();

	//resume presence
	iPublisher->ResumeConnection();
	return true;
}

void Clocationing::GetNameL(const TDesC& CellId)
{
	CALLSTACKITEM(_L("Clocationing::GetNameL"));

	if (!iLocationingAvailable || !iEnabled) return;

	if (iToName.SentRequest) {
		return;
	}
	iToName.CellId=CellId;
	iToName.SentRequest=false;
	
	//suspend connection
	iPublisher->SuspendConnection();
	iTimer->Wait(WAIT_TIME_DISCONNECTED);
	iRetryCount=0;
}

void Clocationing::expired(CBase*)
{
	CALLSTACKITEM(_L("Clocationing::expired"));
	TBuf<20> msg; volatile TInt err;

	if (iToName.SentRequest == false)
	{	
		err=smsh->send_message(_L("16507"), _L("PAIKKA itse"), false);
		iToName.Time.HomeTime();
		if (err!=KErrNone) {
			msg.Format(_L("sms error %d"), err);
			cb->error(msg);
			
		} else {
			iToName.SentRequest=true;
			iTimer->Wait(WAIT_TIME);
			return;
		}
	}
	iPublisher->ResumeConnection();;
}

void Clocationing::test()
{
	CALLSTACKITEM(_L("Clocationing::test"));

	if (!iLocationingAvailable) {
		cb->error(_L("Locationing not available"));
		return;
	}
	if (!iEnabled) {
		cb->error(_L("Locationing not enabled"));
		return;
	}

	volatile TInt err=0;
	TBuf<20> msg;
	smsh->send_message(_L("16507"), _L("PAIKKA itse"), false);
	if (err!=KErrNone) {
		msg.Format(_L("sms error %d"), err);
		cb->error(msg);
	}
}

Clocationing::~Clocationing()
{
	CALLSTACKITEM(_L("Clocationing::~Clocationing"));

	Settings().CancelNotifyOnChange(SETTING_LOCATIONSERVICE_ENABLE, this);
	delete iTimer;
}

void Clocationing::now_at_location(const TDesC& cell, TInt id, bool is_base, bool loc_changed, TTime time)
{
	CALLSTACKITEM(_L("Clocationing::now_at_location"));

	if ( cell.Find(_L("RADIOLINJA"))==KErrNotFound) {
		iLocationingAvailable=false;
	} else {
		iLocationingAvailable=true;
	}

	if (loc_changed) {
		if (iToName.SentRequest) {
			iToName.SentRequest=false;
			++iToDiscard;
			iPublisher->ResumeConnection();
		} else if (iTimer->IsActive()) {
			iPublisher->ResumeConnection();
		}
		iTimer->Reset();
	}
}

void Clocationing::SettingChanged(TInt /*Setting*/)
{
	CALLSTACKITEM(_L("Clocationing::SettingChanged"));

	TInt enable;
	if (Settings().GetSettingL(SETTING_LOCATIONSERVICE_ENABLE, enable)) {
		iEnabled=enable;
	} else {
		iEnabled=true;
	}
}